@@ -1,5 +1,22 @@
-# Installation prefix information
-import prefix
+"""
+Copyright (c) 2003-2007, Paul Varga and other contributors
+All rights reserved.
+
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions are met:
+
+    * Redistributions of source code must retain the above copyright notice,
+      this list of conditions and the following disclaimer.
+    * Redistributions in binary form must reproduce the above copyright notice,
+      this list of conditions and the following disclaimer in the documentation
+      and/or other materials provided with the distribution.
+    * Neither the name of the project nor the names of its contributors may be
+      used to endorse or promote products derived from this software without
+      specific prior written permission.
+"""
+
+# gnome-btdownload stuff
+import formattime, prefix, progressicon
 
 # BitTorrent related modules.
 import BitTorrent.download, BitTorrent.bencode
@@ -16,15 +33,21 @@ except:
 	import gnome.vfs
 	gnomevfs = gnome.vfs
 
+# pynotify
+import pynotify
+
 # Gettext
 import gettext
 _ = gettext.gettext
 
 # The name of this program.
-app_name         = 'gnome-btdownload'
+app_name    = 'gnome-btdownload'
 
 # The version of this program.
-app_version      = '0.0.28'
+app_version = '0.0.30'
+
+# The 'GnomeProgram' for this process.
+gnome_program = gnome.program_init(app_name, app_version)
 
 # A hack that is set to a value that is the largest possible BitTorrent meta
 # file. This is passed to get_url to pull the entire (hopefully) meta file into
@@ -47,8 +70,35 @@ rfc_2396_uri_regexp = r"^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))
 #		A potential sub-directory to check (and prefer) for the file.
 locate_file = None
 
-# The 'GnomeProgram' for this process.
-gnome_program = gnome.program_init(app_name, app_version)
+# Disabled until gnome_program.locate_file is exported by gnome-python...
+#def locate_file(type filename, sub):
+#	FIXME gnome_program.locate_file(gnome.FILE_DOMAIN_APP_DATADIR, filename, True)
+
+# Fallback wrapper
+if not locate_file:
+	def fallback_locate_attempt(p, path, sub, filename):
+		if sub:
+			p_path_sub_file = p + '/' + path + '/' + sub + '/' + filename
+			if os.path.exists(p_path_sub_file):
+				return p_path_sub_file
+		
+		p_path_file = p + '/' + path + '/' + filename
+		if os.path.exists(p_path_file):
+			return p_path_file
+		
+		return None
+	
+	def fallback_locate_file(type, filename, sub=None):
+		if type == 'data':
+			result = fallback_locate_attempt(prefix.datadir, app_name, sub, filename)
+			if result:
+				return result
+		
+		print >> sys.stderr, _("Couldn't locate file, will probably explode...")
+		return None
+	
+	if not locate_file:
+		locate_file = fallback_locate_file
 
 # Makes sure a URI is fully qualified and return the result. Return None if it
 # isn't a URI.
@@ -61,50 +111,31 @@ def fix_up_uri(path):
 	except:
 		return None
 
-# Checks if a path exists. This is still around from when this was a 'virtual'
-# function.
-def path_exists(path):
-	return gnomevfs.exists(path)
-
-# GNOME wrapper
-def can_show_path(path):
-	return True
-def show_path(path):
-	gnome.url_show(str(fix_up_uri(path)))
-
-# GNOME wrapper
-def get_home_dir():
-	return os.path.expanduser('~')
-
 # Wrapper
 def get_config_dir():
-	home_dir = get_home_dir()
+	home_dir = os.path.expanduser('~')
 	
-	if path_exists(os.path.join(home_dir, '.gnome2')):
+	if gnomevfs.exists(os.path.join(home_dir, '.gnome2')):
 		return os.path.join(home_dir, '.gnome2', app_name)
 	else:
 		return os.path.join(home_dir, '.'+app_name)
 def make_config_dir():
-	home_dir   = get_home_dir()
+	home_dir   = os.path.expanduser('~')
 	config_dir = None
 	
-	if path_exists(os.path.join(home_dir, '.gnome2')):
+	if gnomevfs.exists(os.path.join(home_dir, '.gnome2')):
 		config_dir = os.path.join(home_dir, '.gnome2', app_name)
 	else:
 		config_dir = os.path.join(home_dir, '.'+app_name)
 	
-	if not path_exists(config_dir):
+	if not gnomevfs.exists(config_dir):
 		gnomevfs.make_directory(config_dir, 00750)
 	
 	cache_dir = os.path.join(config_dir, 'cache')
 	
-	if not path_exists(cache_dir):
+	if not gnomevfs.exists(cache_dir):
 		gnomevfs.make_directory(cache_dir, 00777)
 
-# Disabled until gnome_program.locate_file is exported by gnome-python...
-#def locate_file(type filename, sub):
-#	FIXME gnome_program.locate_file(gnome.FILE_DOMAIN_APP_DATADIR, filename, True)
-
 # Load at most read_bytes from a URI and return the result.
 def get_url(uri, read_bytes):
 	content = ''
@@ -138,133 +169,9 @@ def idle_add_once(function, *args):
 		return False
 	gobject.idle_add(function_false, function, *args)
 
-# Fallback wrapper
-if not locate_file:
-	def fallback_locate_attempt(p, path, sub, filename):
-		if sub:
-			p_path_sub_file = p + '/' + path + '/' + sub + '/' + filename
-			if os.path.exists(p_path_sub_file):
-				return p_path_sub_file
-		
-		p_path_file = p + '/' + path + '/' + filename
-		if os.path.exists(p_path_file):
-			return p_path_file
-		
-		return None
-	
-	def fallback_locate_file(type, filename, sub=None):
-		if type == 'data':
-			result = fallback_locate_attempt(prefix.datadir, app_name, sub, filename)
-			if result:
-				return result
-		
-		print >> sys.stderr, _("Couldn't locate file, will probably explode...")
-		return None
-	
-	if not locate_file:
-		locate_file = fallback_locate_file
-
-# Converts a number of seconds into a short displayable string.
-def fmt_time_short(seconds):
-	seconds = int(seconds)
-	
-	hours    = seconds / (60 * 60)
-	seconds -= hours * (60 * 60)
-	
-	minutes  = seconds / 60
-	seconds -= minutes * 60
-	
-	return '%0.2d:%0.2d:%0.2d' % (hours, minutes, seconds)
-
-# Converts a number of seconds into a precise, but verbose displayable string.
-def fmt_time_long_precise_verbose(seconds):
-	seconds = int(seconds)
-	
-	days     = seconds / (60 * 60 * 24)
-	seconds -= days * (60 * 60 * 24)
-	
-	hours    = seconds / (60 * 60)
-	seconds -= hours * (60 * 60)
-	
-	minutes  = seconds / 60
-	seconds -= minutes * 60
-	
-	# FIXME Kind-of convoluted, not really locale friendly...
-	def create_listing(items):
-		def append_listing(listing, singular, plural, count, index, length):
-			initial = not listing
-			
-			if listing or count != 0 or index+1 == length:
-				listing += str(count) + ' '
-				
-				if count == 1:
-					listing += singular
-				else:
-					listing += plural
-				
-				
-				if index+1 < length:
-					if index+2 == length:
-						if initial:
-							listing += ' and '
-						else:
-							listing += ', and '
-					else:
-						listing += ', '
-			
-			return listing
-		
-		listing = ''
-		
-		for index, item in zip(range(0,len(items)), items):
-			listing = append_listing(listing, item[0][0], item[0][1], item[1], index, len(items))
-		
-		return listing
-	
-	return create_listing((
-		(('day', 'days'), days),
-		(('hour', 'hours'), hours),
-		(('minute', 'minutes'), minutes),
-		(('second', 'seconds'), seconds)
-	))
-
-# Converts a number of seconds into a rough estimate
-def fmt_time_long_estimate(seconds):
-	seconds = int(seconds)
-	
-	days     = seconds / (60 * 60 * 24)
-	seconds -= days * (60 * 60 * 24)
-	
-	hours    = seconds / (60 * 60)
-	seconds -= hours * (60 * 60)
-	
-	minutes  = seconds / 60
-	seconds -= minutes * 60
-	
-	if days > 1:
-		return _("About %i days") % days
-	elif days == 1:
-		return _("About %i day") % days
-	elif hours > 1:
-		return _("About %i hours") % hours
-	elif hours == 1:
-		return _("About %i hour") % hours
-	elif minutes > 1:
-		return _("About %i minutes") % minutes
-	elif minutes == 1:
-		return _("About %i minute") % minutes
-	elif seconds > 1:
-		return _("About %i seconds") % seconds
-	elif seconds == 1:
-		return _("About %i second") % seconds
-	elif seconds <= 0:
-		return _("About %i seconds") % seconds
-
-fmt_time_long = fmt_time_long_estimate
-
 # A GNOME HIG compliant error dialog wrapper.
 class GtkHigErrorDialog(gtk.MessageDialog):
-	def __init__(self, text, subtext='', modal=False, parent=None):
+	def __init__(self, text, subtext=None, modal=False, parent=None):
 		flags = gtk.DIALOG_DESTROY_WITH_PARENT
 		
 		if modal:
@@ -791,17 +698,17 @@ def check_for_previous_save_location(bt_state_args):
 	config_dir = get_config_dir()
 	cache_dir = os.path.join(config_dir, 'cache')
 	
-	if path_exists(cache_dir):
+	if gnomevfs.exists(cache_dir):
 		torrent_file = bt_state_args.get_torrent_file()
 		
 		if torrent_file:
 			digest     = sha.new(torrent_file).hexdigest()
 			digest_url = os.path.join(cache_dir, digest)
 			
-			if path_exists(digest_url):
+			if gnomevfs.exists(digest_url):
 				previous_save_location = get_url(digest_url, max_torrent_size)
 				
-				if path_exists(fix_up_uri(previous_save_location)):
+				if gnomevfs.exists(fix_up_uri(previous_save_location)):
 					return previous_save_location
 		else:
 			print >> sys.stderr, 'check_for_previous_save_location failed to get torrent_file'
@@ -812,7 +719,7 @@ def cache_save_location(bt_state_args):
 	config_dir = get_config_dir()
 	cache_dir = os.path.join(config_dir, 'cache')
 	
-	if not path_exists(cache_dir):
+	if not gnomevfs.exists(cache_dir):
 		make_config_dir()
 	
 	# Just to make sure
@@ -857,10 +764,13 @@ class GtkClient:
 		self.gconf_client.add_dir('/apps/'+app_name, gconf.CLIENT_PRELOAD_RECURSIVE)
 		#self.gconf_client.notify_add('/apps/'+app_name+'/settings', self.on_gconf_settings_notify)
 		
+		# pynotify Setup
+		self.notify_enabled = pynotify.init(app_name)
+		
 		# Bt Setup
 		bt_state_args = BtState.Args(args[1:])
 		
-		if not bt_state_args.path_origin or not path_exists(bt_state_args.path_origin):
+		if not bt_state_args.path_origin or not gnomevfs.exists(bt_state_args.path_origin):
 			filters = ((True, _("BitTorrent meta files"), 'mime', 'application/x-bittorrent'), )
 			result = GtkFileOpenDialog(_("Open location for BitTorrent meta file"), filters=filters, modal=True).run()
 			
@@ -922,14 +832,20 @@ class GtkClient:
 		
 		self.bt_state = BtState(bt_state_args)
 		
+		# StatusIcon Setup
+		self.status_icon          = gtk.StatusIcon()
+		self.status_icon_fraction = None
+		
+		self.status_icon.connect('activate', self.on_status_icon_activate)
+		self.status_icon.connect('popup-menu', self.on_status_icon_popup_menu)
+		self.status_icon.connect('size-changed', self.on_status_icon_size_changed)
+		
 		# Run Gtk+ main window
 		self.glade_xml = gtk.glade.XML(locate_file('data', 'dlsession.glade', 'glade'))
 	
 		self.setup_treeview_events()
 		
 		self.glade_xml.signal_autoconnect({
-			'on_window_main_destroy':
-				self.on_window_main_destroy,
 			'on_window_main_focus_in_event':
 				self.on_window_main_focus_event,
 			'on_window_main_focus_out_event':
@@ -937,13 +853,13 @@ class GtkClient:
 			'on_window_main_destroy':
 				self.on_window_main_destroy,
 			'on_button_open_clicked':
-				self.on_button_open_clicked,
+				lambda widget: self.open(),
 			'on_button_resume_clicked':
-				self.on_button_resume_clicked,
+				lambda widget: self.resume(),
 			'on_button_stop_clicked':
-				self.on_button_stop_clicked,
+				lambda widget: self.stop(),
 			'on_button_close_clicked':
-				self.on_button_close_clicked,
+				lambda widget: self.close(),
 			'on_checkbutton_cap_uploads_toggled':
 				self.on_checkbutton_cap_uploads_toggled,
 			'on_spinbutton_cap_uploads_value_changed':
@@ -954,6 +870,8 @@ class GtkClient:
 				self.on_spinbutton_cap_upload_rate_value_changed,
 			'on_button_events_clear_clicked':
 				self.on_button_events_clear_clicked,
+			'on_checkbutton_events_display_status_icon_toggled':
+				self.on_checkbutton_events_display_status_icon_toggled,
 			'on_checkbutton_events_display_error_dialogs_toggled':
 				self.on_checkbutton_events_display_error_dialogs_toggled
 		})
@@ -962,6 +880,10 @@ class GtkClient:
 		self.glade_xml.get_widget('window_main').set_title(os.path.basename(self.bt_state.path_output))
 		
 		# Set GUI preferences
+		display_status_icon = self.gconf_client.get_bool('/apps/'+app_name+'/settings/display_status_icon')
+		if display_status_icon != None:
+			self.glade_xml.get_widget('checkbutton_events_display_status_icon').set_active(display_status_icon)
+		
 		display_error_dialogs = self.gconf_client.get_bool('/apps/'+app_name+'/settings/display_error_dialogs')
 		if display_error_dialogs != None:
 			self.glade_xml.get_widget('checkbutton_events_display_error_dialogs').set_active(display_error_dialogs)
@@ -1005,38 +927,109 @@ class GtkClient:
 		
 		master_client.set_restart_style(gnome.ui.RESTART_IF_RUNNING)
 		
+		# Setup status icon
+		self.update_status_icon()
+		
 		# Run Bt
 		self.run_bt()
 		
 		# Run Gtk+
 		gtk.main()
 	
+	# Actions
+	def open(self):
+		uri = fix_up_uri(self.bt_state.path_output)
+		try:
+			gnome.url_show(str(uri))
+		except:
+			# If we can't open the file, show it in the file
+			# browser. In Python we cannot access libeel's
+			# open-with dialog, so this is the next best thing.
+			gnome.url_show(str(uri.parent))
+	def resume(self):
+		button_resume = self.glade_xml.get_widget('button_resume')
+		button_stop   = self.glade_xml.get_widget('button_stop')
+		button_close  = self.glade_xml.get_widget('button_close')
+		
+		button_resume.set_sensitive(False)
+		button_stop.show()
+		button_close.hide()
+		
+		self.run_bt(resuming=True)
+	def stop(self):
+		self.join()
+		
+		button_resume = self.glade_xml.get_widget('button_resume')
+		button_stop   = self.glade_xml.get_widget('button_stop')
+		button_close  = self.glade_xml.get_widget('button_close')
+		
+		button_resume.set_sensitive(True)
+		button_stop.hide()
+		button_close.show()
+	def close(self):
+		self.join()
+		
+		window_main = self.glade_xml.get_widget('window_main')
+		window_main.destroy()
+	def show_window_main(self, show):
+		window_main = self.glade_xml.get_widget('window_main')
+		
+		if show:
+			window_main.present()
+		else:
+			window_main.hide()
+	
 	# Appends an event to the log.
 	def log_event(self, type, text):
-		t = fmt_time_short(time.time() - self.bt_state.time_begin)
+		# Even though this is only run in the GUI thread, this seems to
+		# be necessary to prevent the dialog from deadlocking on a
+		# mutex.
+		gtk.gdk.threads_enter()
+		
+		t = formattime.succinct(time.time() - self.bt_state.time_begin)
 		
 		notebook_main = self.glade_xml.get_widget('notebook_main')
 		vbox_events   = self.glade_xml.get_widget('vbox_events')
 		events_tab    = notebook_main.page_num(vbox_events)
 		
-		# Check if the user is looking at the events tab. If so, do not
-		# pop up an error dialog.
-		if events_tab != None and notebook_main.get_current_page() != events_tab:
-			if type == 'Error':
-				# Select the events tab now (this will also prevent
-				# more error dialogs from popping up).
-				#
-				# Say it with me: "worst solution EVER". :P
-				notebook_main.set_current_page(events_tab)
+		if type == 'Error':
+			primary, secondary = str(text), None
+			
+			# Try to specially adapt the error message.
+			try:
+				mo = re.search(r"([A-Za-z ',]+) - [<]?([^>]+)[>]?", primary)
+				primary, secondary = mo.group(1), mo.group(2)
+			except:
+				pass
+			
+			# Should we silently log errors or no?
+			if self.glade_xml.get_widget('checkbutton_events_display_error_dialogs').get_active():
+				done = False
 				
-				if self.glade_xml.get_widget('checkbutton_events_display_error_dialogs').get_active():
-					# Try to specially adapt the error message.
-					try:
-						mo = re.search(r"([A-Za-z ',]+) - [<]?([^>]+)[>]?", str(text))
+				# Check if the status icon is showing and notify is
+				# enabled. If so, try to use that for errors.
+				if self.status_icon.get_visible() and self.notify_enabled:
+					markup = primary
+					if secondary is not None:
+						markup = '<b>%s</b>\n\n%s' % (primary, secondary)
 					
-						GtkHigErrorDialog(mo.group(1), mo.group(2), parent=self.glade_xml.get_widget('window_main')).run()
-					except:
-						GtkHigErrorDialog(str(text), parent=self.glade_xml.get_widget('window_main')).run()
+					n = pynotify.Notification(markup)
+					n.attach_to_status_icon(self.status_icon)
+					n.set_urgency(pynotify.URGENCY_NORMAL)
+					
+					if n.show():
+						done = True
+				if not done:
+					# Check if the user is looking at the events tab. If so, do not
+					# pop up an error dialog.
+					if events_tab != None and notebook_main.get_current_page() != events_tab:
+						# Select the events tab now (this will also prevent
+						# more error dialogs from popping up).
+						#
+						# Say it with me: "worst solution EVER". :P
+						notebook_main.set_current_page(events_tab)
+						
+						GtkHigErrorDialog(primary, secondary, parent=self.glade_xml.get_widget('window_main')).run()
 		
 		if self.bt_events:
 			treeview_events = self.glade_xml.get_widget('treeview_events')
@@ -1046,7 +1039,9 @@ class GtkClient:
 			path = treeview_events.get_model().get_path(iter)
 			treeview_events.scroll_to_cell(path)
 		else:
-			print >> sys.stderr, i18n('%s, %s: %s' % (t, type, text))
+			print >> sys.stderr, i18n('%s, %s: %s' % (t, type, str(text)))
+		
+		gtk.gdk.threads_leave()
 	
 	# BitTorrent callbacks
 	def on_bt_file(self, default, size, saveas, dir):
@@ -1074,17 +1069,20 @@ class GtkClient:
 		self.bt_state.status(dict)
 		
 		label_download_elapsed_output = self.glade_xml.get_widget('label_download_time_elapsed_output')
-		idle_add_once(label_download_elapsed_output.set_text, fmt_time_long_precise_verbose(time.time() - self.bt_state.time_begin))
+		idle_add_once(label_download_elapsed_output.set_text, formattime.lengthy_precise(time.time() - self.bt_state.time_begin))
 		
 		if dict.has_key('fractionDone'):
 			progressbar_download_status = self.glade_xml.get_widget('progressbar_download_status')
 			window_main = self.glade_xml.get_widget('window_main')
 			
-			perc_string = str(int(dict['fractionDone'] * 100)) + '%'
+			fraction = float(dict['fractionDone'])
+			perc_string = str(int(fraction * 100.0)) + '%'
+			title = _("%s of %s") % (perc_string, os.path.basename(self.bt_state.path_output))
 			
 			idle_add_once(progressbar_download_status.set_fraction, dict['fractionDone'])
 			idle_add_once(progressbar_download_status.set_text, perc_string)
-			idle_add_once(window_main.set_title, _("%s of %s") % (perc_string, os.path.basename(self.bt_state.path_output)))
+			idle_add_once(window_main.set_title, title)
+			idle_add_once(self.update_status_icon, title, fraction)
 		
 		if dict.has_key('downTotal') or dict.has_key('fractionDone'):
 			label_download_status_output = self.glade_xml.get_widget('label_download_status_output')
@@ -1097,7 +1095,7 @@ class GtkClient:
 		if dict.has_key('timeEst'):
 			label_download_time_remaining_output = self.glade_xml.get_widget('label_download_time_remaining_output')
 			
-			idle_add_once(label_download_time_remaining_output.set_text, fmt_time_long(dict['timeEst']))
+			idle_add_once(label_download_time_remaining_output.set_text, formattime.lengthy(dict['timeEst']))
 		
 		if dict.has_key('upRate') or dict.has_key('upTotal'):
 			label_upload_status_output = self.glade_xml.get_widget('label_upload_status_output')
@@ -1120,7 +1118,10 @@ class GtkClient:
 		label_download_status_output         = self.glade_xml.get_widget('label_download_status_output')
 		progressbar_download_status          = self.glade_xml.get_widget('progressbar_download_status')
 		label_download_time_remaining_output = self.glade_xml.get_widget('label_download_time_remaining_output')
+		button_close                         = self.glade_xml.get_widget('button_close')
 		button_open                          = self.glade_xml.get_widget('button_open')
+		button_resume                        = self.glade_xml.get_widget('button_resume')
+		button_stop                          = self.glade_xml.get_widget('button_stop')
 		window_main                          = self.glade_xml.get_widget('window_main')
 		
 		idle_add_once(label_download_status_output.set_text, _("%.1f of %.1f MB at %.2f KB/s") %
@@ -1131,23 +1132,17 @@ class GtkClient:
 		idle_add_once(progressbar_download_status.set_text, '100%')
 		idle_add_once(label_download_time_remaining_output.set_text, _("None"))
 		
-		try:
-			# Try to set the urgency hint if the user isn't paying adequite attention
-			if not window_main.has_toplevel_focus():
-				idle_add_once(window_main.set_urgency_hint, True)
-		except:
-			pass
+		# Update buttons
+		idle_add_once(button_resume.hide)
+		idle_add_once(button_stop.hide)
+		idle_add_once(button_close.show)
+		idle_add_once(button_open.show)
 		
-		# Check if the completed session can be 'shown'
-		if can_show_path and show_path and can_show_path(self.bt_state.path_output):
-			idle_add_once(button_open.set_sensitive, True)
+		# Notify of completion
+		idle_add_once(self.notify_of_completion)
 	
 	def on_bt_error(self, msg):
-		gtk.gdk.threads_enter()
-		
-		self.log_event('Error', msg)
-		
-		gtk.gdk.threads_leave()
+		idle_add_once(self.log_event, 'Error', msg)
 	
 	def on_bt_path(self, path):
 		self.bt_state.path(path)
@@ -1207,45 +1202,16 @@ class GtkClient:
 		gtk.main_quit()
 	
 	# GTK+ callbacks
-	def on_window_main_destroy(self, widget, data=None):
-		self.join()
-		gtk.main_quit()
 	def on_window_main_focus_event(self, widget, event):
 		# Try to unset the urgency event
 		try:
 			widget.set_urgency_hint(False)
 		except:
 			pass
-	
-	def on_button_open_clicked(self, widget, data=None):
-		if show_path:
-			show_path(self.bt_state.path_output)
-	
-	def on_button_resume_clicked(self, widget, data=None):
-		button_resume = self.glade_xml.get_widget('button_resume')
-		button_stop   = self.glade_xml.get_widget('button_stop')
-		button_close  = self.glade_xml.get_widget('button_close')
-		
-		button_resume.set_sensitive(False)
-		button_stop.show()
-		button_close.hide()
-		
-		self.run_bt(resuming=True)
-	
-	def on_button_stop_clicked(self, widget, data=None):
+		return False
+	def on_window_main_destroy(self, widget, data=None):
 		self.join()
-		
-		button_resume = self.glade_xml.get_widget('button_resume')
-		button_stop   = self.glade_xml.get_widget('button_stop')
-		button_close  = self.glade_xml.get_widget('button_close')
-		
-		button_resume.set_sensitive(True)
-		button_stop.hide()
-		button_close.show()
-	
-	def on_button_close_clicked(self, widget, data=None):
-		window_main = self.glade_xml.get_widget('window_main')
-		window_main.destroy()
+		gtk.main_quit()
 	
 	def on_checkbutton_cap_uploads_toggled(self, widget, data=None):
 		spinbutton_cap_uploads = self.glade_xml.get_widget('spinbutton_cap_uploads')
@@ -1299,14 +1265,120 @@ class GtkClient:
 		if self.bt_events:
 			self.bt_events.clear()
 	
-	def on_checkbutton_events_display_error_dialogs_toggled(self, widget, data=None):
+	def on_checkbutton_events_display_status_icon_toggled(self, widget, data=None):
 		try:
-			if widget.get_active():
-				self.gconf_client.set_bool('/apps/'+app_name+'/settings/display_error_dialogs', True)
-			else:
-				self.gconf_client.set_bool('/apps/'+app_name+'/settings/display_error_dialogs', False)
+			self.gconf_client.set_bool('/apps/'+app_name+'/settings/display_status_icon', widget.get_active())
 		except:
 			pass
+		self.update_status_icon()
+	def on_checkbutton_events_display_error_dialogs_toggled(self, widget, data=None):
+		try:
+			self.gconf_client.set_bool('/apps/'+app_name+'/settings/display_error_dialogs', widget.get_active())
+		except:
+			pass
+	
+	def on_status_icon_activate(self, status_icon, data=None):
+		window_main = self.glade_xml.get_widget('window_main')
+		
+		self.show_window_main(not window_main.get_property('visible'))
+	def on_status_icon_popup_menu(self, status_icon, button, activate_time, data=None):
+		screen, area, orientation = status_icon.get_geometry()
+		if status_icon.is_embedded() and area is not None:
+			def new_entry(stock, markup):
+				menuitem = gtk.ImageMenuItem(stock)
+				menuitem.get_child().set_markup_with_mnemonic(markup)
+				menuitem.show()
+				return menuitem
+			
+			button_resume = self.glade_xml.get_widget('button_resume')
+			button_stop   = self.glade_xml.get_widget('button_stop')
+			button_close  = self.glade_xml.get_widget('button_close')
+			window_main   = self.glade_xml.get_widget('window_main')
+			checkbutton_events_display_error_dialogs = self.glade_xml.get_widget('checkbutton_events_display_error_dialogs')
+			
+			menu = gtk.Menu()
+			
+			if button_resume.get_property('visible') and button_resume.get_property('sensitive'):
+				resume = new_entry(gtk.STOCK_REFRESH, _('_Resume'))
+				resume.connect('activate', lambda widget: self.resume())
+				menu.add(resume)
+			
+			if button_stop.get_property('visible'):
+				stop = new_entry(gtk.STOCK_STOP, _('_Stop'))
+				stop.connect('activate', lambda widget: self.stop())
+				menu.add(stop)
+			
+			if button_close.get_property('visible'):
+				close = new_entry(gtk.STOCK_CLOSE, _('_Close'))
+				close.connect('activate', lambda widget: self.close())
+				menu.add(close)
+			
+			menuitem = gtk.SeparatorMenuItem()
+			menuitem.show()
+			menu.add(menuitem)
+			
+			menuitem = gtk.CheckMenuItem(_('Display BitTorrent Session'))
+			menuitem.set_active(window_main.get_property('visible'))
+			menuitem.connect('toggled', lambda menuitem: self.show_window_main(menuitem.get_active()))
+			menuitem.show()
+			menu.add(menuitem)
+			
+			menuitem = gtk.CheckMenuItem(_('Display Error Dialogs'))
+			menuitem.set_active(checkbutton_events_display_error_dialogs.get_active())
+			menuitem.connect('toggled', lambda menuitem: checkbutton_events_display_error_dialogs.set_active(menuitem.get_active()))
+			menuitem.show()
+			menu.add(menuitem)
+			
+			menu.popup(None, None, gtk.status_icon_position_menu, button, activate_time, status_icon)
+	def on_status_icon_size_changed(self, status_icon, size, data=None):
+		self.update_status_icon()
+	
+	def notify_of_completion(self):
+		done = False
+		
+		# If the status icon is displayed and notification works, notify the user of completion
+		if self.status_icon.get_visible() and self.notify_enabled:
+			n = pynotify.Notification('Download Complete')
+			n.attach_to_status_icon(self.status_icon)
+			n.set_urgency(pynotify.URGENCY_LOW)
+			
+			if n.show():
+				done = True
+		
+		# Otherwise, try to set the urgency hint if the user isn't paying adequite attention
+		if not done:
+			try:
+				if not window_main.has_toplevel_focus():
+					idle_add_once(window_main.set_urgency_hint, True)
+			except:
+				pass
+	
+	def update_status_icon(self, title=None, fraction=None):
+		checkbutton_events_display_status_icon = self.glade_xml.get_widget('checkbutton_events_display_status_icon')
+		
+		if checkbutton_events_display_status_icon.get_active():
+			granularity = 0.2
+			
+			if self.status_icon_fraction is None or fraction is None or self.status_icon_fraction != int(fraction / granularity)*granularity:
+				if fraction is None:
+					if self.status_icon_fraction is None:
+						self.status_icon_fraction = 0.0
+				else:
+					self.status_icon_fraction = int(fraction / granularity)*granularity
+				
+				progress_icon = progressicon.render_progress_icon(
+					locate_file('data', 'download.svg', 'pixmaps'),
+					self.status_icon.get_size(), self.status_icon.get_size(),
+					self.status_icon_fraction)
+				
+				self.status_icon.set_from_pixbuf(progress_icon)
+			
+			if title is not None:
+				self.status_icon.set_tooltip(title)
+			
+			self.status_icon.set_visible(True)
+		else:
+			self.status_icon.set_visible(False)
 	
 	# GTK+ setup stuff to supliment Glade.
 	def setup_treeview_events(self):
@@ -1333,5 +1405,5 @@ class GtkClient:
 
 # Start the client
 def run(args):
-	gtk.window_set_default_icon_from_file(locate_file('data', 'download.png', 'pixmaps'))
+	gtk.window_set_default_icon_from_file(locate_file('data', 'download.svg', 'pixmaps'))
 	client = GtkClient(args)
